# La empresa Jacuzzis Rivera (JR) fabrica tres productos:
# Tina Básica (X1), Tina Chevere (X2) y Tina De Lujo (X3).
# Para el siguiente mes tiene disponibles cantidades
# limitadas de los recursos que usa (bombas de agua,
# horas de mano de obra y pies de tubería). Deseamos
# fabricar la cantidad de unidades de cada producto
# que maximice la utilidad total de la empresa. 

# MAX Z = 350 X1 + 300 X2 + 320 X3
# Subject to:
#  1 X1 + 1  X2 +  1 X3 ≤ 200       Bombas
#  9 X1 +  6 X2 +  8 X3 ≤ 1,566     Horas MdO
# 12 X1 + 16 X2 + 13 X3 ≤ 2,880     Tubería

using JuMP
using GLPK
using MathOptInterface
using DelimitedFiles

inicio = time()

# Leemos los datos de utilidades de los productos
ut = readdlm("Utilidades.txt")

# Leemos la disponibilidad de los recursos
disp = readdlm("Disponibles.txt")

# Leemos los datos del consumo de recursos
# para hacer cada producto
cons = readdlm("Consumos.txt")

# Determinamos el número de productos y de recursos
pd = size(ut)[1]
rec = size(disp)[1]

# Creamos el modelo
m = Model(with_optimizer(GLPK.Optimizer))

# Creamos las variables
@variable(m, X[1:pd] >= 0)

# Creamos la función objetivo
@objective(m, Max, sum(X[i]*ut[i] for i in 1:pd))

# Agregamos las restricciones
@constraint(m, caprec[j = 1:rec], sum(X[i]*cons[j,i] for i in 1:pd) <= disp[j])

print(m)

status = optimize!(m)

println("Utilidad total  : ", JuMP.objective_value(m))

for i in 1:pd
    println(X[i],  "            : ", JuMP.value(X[i]))
end

final = time()

println("El tiempo de ejecución fue de ", (final - inicio), " segundos")

# El tiempo de ejecución fue de  segundos



