# Modelo TSP25 de Juan Pablo Orejuela

# Importamos los módulos necesarios
import time
import numpy as np
import math
from python_tsp.exact import solve_tsp_dynamic_programming

# Función para calcular las distancias
# entre los nodos
def eucdist(coords):
    """Recibe un vector de numpy
       con las coordenadas de 
       los nodos. Regresa una 
       matriz de numpy con
       las distancias.
    """
    nodos = len(coords) 
    distancias = np.zeros((nodos,nodos))
    for i in range(nodos):
        for j in range(i+1, nodos):
            distance = math.dist(coords[i][:1], coords[j][:1])
            distancias[i][j] = distance
            distancias[j][i] = distance
    return distancias

# INICIA PROGRAMA PRINCIPAL
def main():
    start = time.time()
    # Leemos los datos de las coordenadas    
    coords = np.loadtxt("TSP38.txt")
    
    nodos = range(len(coords))
    st = range(1,len(coords))
    
    # Creamos la matriz de distancias entre todos los nodos
    distancias = eucdist(coords)
    
    # Resvolvemos el problema con un
    # algoritmo de Programación Dinámica
    permutation, distance = solve_tsp_dynamic_programming(distancias)
    
    print("Orden:     ",permutation, "\n")
    print("Distancia: ",distance) 
    
    end = time.time()

    #Termina la medición de tiempo
    print("El tiempo de ejecución fue de ", (end - start), "segundos")

    # El tiempo de ejecución fue de 3.4899184703826904  segundos

if __name__ == "__main__":
    main()


