# Modelo TSP25 de Juan Pablo Orejuela

# Importamos los módulos necesarios
using JuMP
using Cbc
using MathOptInterface
using DelimitedFiles


# Función para calcular las distancias
# entre los nodos

function eucdist(p1)
    nodos = size(p1)[1]
    distancias = zeros(nodos,nodos)
    for i=1:nodos, j=i+1:nodos
        dist = sqrt((p1[i,1] - p1[j,1])^2 + (p1[i,2] - p1[j,2])^2)
        distancias[i,j] = dist
        distancias[j,i] = dist
    end
    
    return distancias
end


# INICIA PROGRAMA PRINCIPAL
# Iniciamos el cronometro
inicio = time()

# Leemos las coordenadas de los nodos
p1 = readdlm("DJIB38.txt")

# Creamos la matriz de distancias entre nodos
distancias = eucdist(p1)

# Determinamos la cantidad de nodos
nodos = size(p1)[1]

# Creamos el modelo
m = Model(with_optimizer(Cbc.Optimizer))

# Creamos las variables binarias
@variable(m, X[i = 1:nodos, j = 1:nodos], Bin)

# Creamos las variables para los subtours
@variable(m, U[2:nodos] >= 0)

# Creamos la función objetivo
@objective(m, Min, sum(X[i,j] * distancias[i,j] for i in 1:nodos, j in 1:nodos if i != j))

# Agregamos las restricciones
# entrada_nodos
@constraint(m, entnod[j = 1:nodos], sum(X[i,j] for i in 1:nodos if i != j) == 1)
# salida_nodos
@constraint(m, salnod[i = 1:nodos], sum(X[i,j] for j in 1:nodos if i != j) == 1)

# subtours
@constraint(m, st[i = 2:nodos, j = 2:nodos; i != j], U[i] - U[j] + nodos * X[i,j] <= nodos - 1) 

#for i in 2:nodos, j in 2:nodos
#    @constraint(m, U[i] - U[j] + nodos * X[i,j] <= nodos - 1)
#end

print(m)

# Se resuelve el modelo
status = optimize!(m)

# Imprimimos la funcion objetivo
println("Distancia Total      : ", JuMP.objective_value(m))

# Imprimimos las variables
println("VARIABLES DE DECISIÓN: ")
display(JuMP.value.(X))

#Termina la medición de tiempo
final = time()

println("\nEl tiempo de ejecución fue de ", (final - inicio), "segundos")

# El tiempo de ejecución fue de XX  segundos



